package com.yc.ship.module.resource.controller.admin.driver.vo;

import com.yc.ship.framework.common.pojo.PageParam;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import java.time.LocalDateTime;

import static com.yc.ship.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;


@Schema(description = "管理后台 - 驾驶员管理分页 Request VO")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class DriverPageReqVO extends PageParam {
    @Schema(description = "id", example = "11")
    private String id;

    @Schema(description = "驾驶员姓名", example = "张三")
    private String name;

    @Schema(description = "驾驶员编号")
    private String driverCode;

    @Schema(description = "驾驶员分组")
    private String groupId;

    @Schema(description = "身份证号")
    private String idCard;

    @Schema(description = "性别")
    private Integer sex;

    @Schema(description = "生日")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDateTime[] birthday;

    @Schema(description = "联系电话")
    private String phone;

    @Schema(description = "准驾车型(A1,A2,A3...)")
    private Integer driverLevel;

    @Schema(description = "驾驶员类型(0内部、1外部,与供应商类型保持一致)", example = "1")
    private Integer driverType;

    @Schema(description = "学历(小学、初中、高中、大学、研究生..)", example = "2")
    private Integer eduType;

    @Schema(description = "民族(字典选择)")
    private Integer nation;

    @Schema(description = "联系地址")
    private String address;

    @Schema(description = "紧急联系电话")
    private String emergentPhone;

    @Schema(description = "启用状态：0禁用、1启用", example = "1")
    private Integer useStatus;

    @Schema(description = "标签")
    private String driverTag;

    @Schema(description = "驾照正面")
    private String driverCardImgUrl1;

    @Schema(description = "驾照副页")
    private String driverCardImgUrl2;

    @Schema(description = "身份证证件号正面")
    private String idCardImgUrl1;

    @Schema(description = "身份证证件号背面")
    private String idCardImgUrl2;

    @Schema(description = "初次领证日期")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDateTime[] firstIssuedate;

    @Schema(description = "审验到期日期")
    private LocalDateTime annualPeriod;

    @Schema(description = "创建时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDateTime[] createTime;

    @Schema(description = "供应商类型", example = "1")
    private Integer supplierType;

    @Schema(description = "所属供应商id")
    private String companyId;

    @Schema(description = "备注")
    private String remark;

}